/**********************************************************************
Copyright (c) 2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 ***********************************************************************/

package net.sf.mvc.prototype;

import java.awt.BorderLayout;
import java.awt.Frame;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

/**
 * 
 * 
 * {@code SwingUtils} provides static helper methods for all king of
 * swing-related stuff.
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-10-31
 * 
 */
public class SwingUtils {

	private SwingUtils() {
		// singleton
	}

	/**
	 * 
	 * 
	 * Find the currently active Frame.
	 * 
	 * @return the currently active Frame
	 */
	public static Frame findActiveFrame() {
		final Frame[] frames = JFrame.getFrames();
		for (Frame f : frames) {
			if (f.isVisible()) {
				return f;
			}
		}
		return null;
	}

	/**
	 * 
	 * 
	 * Show a message dialog.
	 * 
	 * @param title
	 *            title of the message
	 * @param message
	 *            the message
	 */
	public static void showErrorMessage(String title, String message) {
		JOptionPane.showMessageDialog(findActiveFrame(), message, title,
				JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * 
	 * 
	 * Show a info dialog.
	 * 
	 * @param title
	 *            title of the message
	 * @param message
	 *            the message
	 */
	public static void showInfoMessage(String title, String message) {
		JOptionPane.showMessageDialog(findActiveFrame(), message, title,
				JOptionPane.INFORMATION_MESSAGE);
	}

	/**
	 * 
	 * 
	 * Retrieve a new {@link JPanel} that is initialized with a
	 * {@link BorderLayout} using given vertical- and horizontal gaps.
	 * 
	 * @param vGap
	 *            the vertical gap
	 * @param hGap
	 *            the horizontal gap
	 * @return the new {@link JPanel}
	 */
	public static JPanel getNewPanel(int vGap, int hGap) {
		return new JPanel(new BorderLayout(vGap, hGap));
	}

	/**
	 * 
	 * 
	 * Retrieve a new {@link JPanel} that is initialized with a
	 * {@link BorderLayout} using vertical- and horizontal gaps of 0,0.
	 * 
	 * @return the new {@link JPanel}
	 */
	public static JPanel getNewPanel() {
		return getNewPanel(0, 0);
	}

	/**
	 * 
	 * 
	 * Run a {@link Runnable} in the AWT-event thread, which itself was started
	 * within a new {@link ExceptionGroup}.
	 * 
	 * @param run
	 *            {@link Runnable} to run
	 */
	public final static void run(final Runnable run) {
		ThreadGroup exceptionThreadGroup = new ExceptionGroup("My group");
		new Thread(exceptionThreadGroup, "Init thread") {
			public void run() {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						run.run();
					}
				});
			}
		}.start();
	}
	
	/**
	 * 
	 * Set native L&F
	 *
	 */
	public static void setNativeLookAndFeel() {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println(e);
		}
	}
}
