/**********************************************************************
Copyright (c) 2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 ***********************************************************************/

package net.sf.mvc.prototype.view;

import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.JPanel;

import net.sf.mvc.prototype.controller.InputController;
import net.sf.mvc.prototype.controller.OutputController;
import net.sf.mvc.prototype.model.Model;

/**
 * 
 * An {@code AbstractSwingView} is a prototype implementation of
 * {@link AbstractInputOutputView}, that provides rudimentary
 * {@link javax.swing.JFrame JFrame} functionality.
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-11-01
 * 
 * @param <D>
 *            type of {@link net.sf.mvc.prototype.controller.InputController
 *            InputController}
 *            
 * @see javax.swing.JFrame
 */
public abstract class AbstractSwingView<D extends InputController<? extends Model<? extends OutputController<? extends OutputView>>>>
		extends AbstractInputOutputView<D> {

	/**
	 * This {@code AbstractSwingView}'s frame.
	 */
	protected final JFrame frame = new JFrame();

	/**
	 * This {@code AbstractSwingView}'s content pane.
	 */
	protected final JPanel panel = new JPanel();

	/**
	 * This {@code AbstractSwingView}'s default minimum and preferred size.
	 */
	protected Dimension size = new Dimension(100, 100);

	/**
	 * 
	 * Create a new {@code AbstractSwingView}.
	 * 
	 */
	public AbstractSwingView() {
		init();
	}

	/**
	 * 
	 * Init this {@code AbstractSwingView}. Override this method to make any
	 * changes to this {@code AbstractSwingView} content and/ or behavior.
	 * 
	 */
	protected void init() {
		panel.setOpaque(true);
		frame.setContentPane(panel);
		panel.setMinimumSize(size);
		panel.setPreferredSize(size);
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				for (D c : controllerIn) {
					c.shutdownRequest();
				}
			}
		});
		frame.pack();
	}

	/**
	 * 
	 */
	public void showView() {
		frame.setVisible(true);
	}

	/**
	 * 
	 */
	public void hideView() {
		frame.setVisible(false);
	}

	/**
	 * 
	 */
	public void destroyView() {
		frame.dispose();
	}

	/**
	 * 
	 */
	public void setEnabled(boolean enabled) {
		frame.setEnabled(enabled);
	}

}
